@@ -1,7 +1,7 @@
 package DBIx::Custom;
 use Object::Simple -base;
 
-our $VERSION = '0.1747';
+our $VERSION = '0.2100';
 use 5.008001;
 
 use Carp 'croak';
@@ -19,6 +19,7 @@ use DBIx::Custom::NotExists;
 use Encode qw/encode encode_utf8 decode_utf8/;
 use Scalar::Util qw/weaken/;
 
+
 has [qw/connector dsn password quote user exclude_table user_table_info
         user_column_info/],
     cache => 0,
@@ -129,18 +130,7 @@ sub assign_clause {
     my ($self, $param, $opts) = @_;
     
     my $wrap = $opts->{wrap} || {};
-    my $safety = $self->{safety_character} || $self->safety_character;
-    my $qp = $self->q('');
-    my $q = substr($qp, 0, 1) || '';
-    my $p = substr($qp, 1, 1) || '';
-    
-    # Check unsafety keys
-    unless ((join('', keys %$param) || '') =~ /^[$safety\.]+$/) {
-        for my $column (keys %$param) {
-            croak qq{"$column" is not safety column name } . _subname
-              unless $column =~ /^[$safety\.]+$/;
-        }
-    }
+    my ($q, $p) = split //, $self->q('');
     
     # Assign clause (performance is important)
     join(
@@ -404,9 +394,18 @@ sub execute {
     }
     else {
         $query = $opt{reuse}->{$sql} if $opt{reuse};
-        $query = $self->_create_query($sql,$opt{after_build_sql} || $opt{sqlfilter})
-          unless $query;
-        $query->statement($opt{statement} || '');
+        unless ($query) {
+            my $safety = $self->{safety_character} || $self->safety_character;
+            # Check unsafety keys
+            unless ((join('', keys %$param) || '') =~ /^[$safety\.]+$/) {
+                for my $column (keys %$param) {
+                    croak qq{"$column" is not safety column name } . _subname
+                      unless $column =~ /^[$safety\.]+$/;
+                }
+            }
+            $query = $self->_create_query($sql,$opt{after_build_sql} || $opt{sqlfilter});
+        }
+        $query->{statement} = $opt{statement} || '';
         $opt{reuse}->{$sql} = $query if $opt{reuse};
     }
         
@@ -414,7 +413,10 @@ sub execute {
     $self->{last_sql} = $query->{sql};
 
     # Return query
-    return $query if $opt{query};
+    if ($opt{query}) {
+        delete $param->{$_} for (@cleanup, @{$opt{cleanup} || []});
+        return $query;
+    };
     
     # Merge query filter(DEPRECATED!)
     $filter ||= $query->{filter} || {};
@@ -425,13 +427,13 @@ sub execute {
 
     # Merge id to parameter
     if (defined $opt{id}) {
-        my $statement = $query->statement;
+        my $statement = $query->{statement};
         warn "execute method id option is DEPRECATED!" unless $statement;
         croak "execute id option must be specified with primary_key option"
           unless $opt{primary_key};
         $opt{primary_key} = [$opt{primary_key}] unless ref $opt{primary_key};
         $opt{id} = [$opt{id}] unless ref $opt{id};
-        for (my $i = 0; $i < @{$opt{primary_key}}; $i++) {
+        for (my $i = 0; $i < @{$opt{id}}; $i++) {
            my $key = $opt{primary_key}->[$i];
            $key = "$main_table.$key" if $statement eq 'update' ||
              $statement eq 'delete' || $statement eq 'select';
@@ -447,26 +449,25 @@ sub execute {
     
     # Type rule
     my $type_filters = {};
-    if ($self->{_type_rule_is_called}) {
-        unless ($opt{type_rule_off}) {
-            my $type_rule_off_parts = {
-                1 => $opt{type_rule1_off},
-                2 => $opt{type_rule2_off}
-            };
-            for my $i (1, 2) {
-                unless ($type_rule_off_parts->{$i}) {
-                    $type_filters->{$i} = {};
-                    my $table_alias = $opt{table_alias} || {};
-                    for my $alias (keys %$table_alias) {
-                        my $table = $table_alias->{$alias};
-                        
-                        for my $column (keys %{$self->{"_into$i"}{key}{$table} || {}}) {
-                            $type_filters->{$i}->{"$alias.$column"} = $self->{"_into$i"}{key}{$table}{$column};
-                        }
+    my $type_rule_off = !$self->{_type_rule_is_called} || $opt{type_rule_off};
+    unless ($type_rule_off) {
+        my $type_rule_off_parts = {
+            1 => $opt{type_rule1_off},
+            2 => $opt{type_rule2_off}
+        };
+        for my $i (1, 2) {
+            unless ($type_rule_off_parts->{$i}) {
+                $type_filters->{$i} = {};
+                my $table_alias = $opt{table_alias} || {};
+                for my $alias (keys %$table_alias) {
+                    my $table = $table_alias->{$alias};
+                    
+                    for my $column (keys %{$self->{"_into$i"}{key}{$table} || {}}) {
+                        $type_filters->{$i}->{"$alias.$column"} = $self->{"_into$i"}{key}{$table}{$column};
                     }
-                    $type_filters->{$i} = {%{$type_filters->{$i}}, %{$self->{"_into$i"}{key}{$main_table} || {}}}
-                      if $main_table;
                 }
+                $type_filters->{$i} = {%{$type_filters->{$i}}, %{$self->{"_into$i"}{key}{$main_table} || {}}}
+                  if $main_table;
             }
         }
     }
@@ -495,25 +496,46 @@ sub execute {
           $filter->{$column} = $self->filters->{$name};
         }
     }
-    
-    # Create bind values
-    my ($bind, $bind_types) = $self->_create_bind_values($param, $query->columns,
-      $filter, $type_filters, $opt{bind_type} || $opt{type} || {});
 
     # Execute
     my $sth = $query->{sth};
     my $affected;
-    eval {
-        if ($opt{bind_type} || $opt{type}) {
-            $sth->bind_param($_ + 1, $bind->[$_],
-                $bind_types->[$_] ? $bind_types->[$_] : ())
-              for (0 .. @$bind - 1);
-            $affected = $sth->execute;
-        }
-        else {
-            $affected = $sth->execute(@$bind);
-        }
-    };
+    if (!$query->{duplicate} && $type_rule_off && !keys %$filter &&
+      !$opt{bind_type} && !$opt{type} && !$ENV{DBIX_CUSTOM_DEBUG})
+    {
+        eval { $affected = $sth->execute(map { $param->{$_} } @{$query->{columns}}) };
+    }
+    else {
+        # Create bind values
+        my ($bind, $bind_types) = $self->_create_bind_values($param, $query->{columns},
+          $filter, $type_filters, $opt{bind_type} || $opt{type} || {});
+
+        # Execute
+        eval {
+            if ($opt{bind_type} || $opt{type}) {
+                $sth->bind_param($_ + 1, $bind->[$_],
+                    $bind_types->[$_] ? $bind_types->[$_] : ())
+                  for (0 .. @$bind - 1);
+                $affected = $sth->execute;
+            }
+            else {
+                $affected = $sth->execute(@$bind);
+            }
+
+            # DEBUG message
+            if ($ENV{DBIX_CUSTOM_DEBUG}) {
+                warn "SQL:\n" . $query->{sql} . "\n";
+                my @output;
+                for my $value (@$bind) {
+                    $value = 'undef' unless defined $value;
+                    $value = encode($ENV{DBIX_CUSTOM_DEBUG_ENCODING} || 'UTF-8', $value)
+                      if utf8::is_utf8($value);
+                    push @output, $value;
+                }
+                warn "Bind values: " . join(', ', @output) . "\n\n";
+            }
+        };
+    }
     
     $self->_croak($@, qq{. Following SQL is executed.\n}
       . qq{$query->{sql}\n} . _subname) if $@;
@@ -521,19 +543,6 @@ sub execute {
     # Remove id from parameter
     delete $param->{$_} for (@cleanup, @{$opt{cleanup} || []});
     
-    # DEBUG message
-    if ($ENV{DBIX_CUSTOM_DEBUG}) {
-        warn "SQL:\n" . $query->sql . "\n";
-        my @output;
-        for my $value (@$bind) {
-            $value = 'undef' unless defined $value;
-            $value = encode($ENV{DBIX_CUSTOM_DEBUG_ENCODING} || 'UTF-8', $value)
-              if utf8::is_utf8($value);
-            push @output, $value;
-        }
-        warn "Bind values: " . join(', ', @output) . "\n\n";
-    }
-    
     # Not select statement
     return $affected unless $sth->{NUM_OF_FIELDS};
 
@@ -552,7 +561,7 @@ sub execute {
     }
     
     # Result
-    my $result = $self->result_class->new(
+    $self->result_class->new(
         sth => $sth,
         dbi => $self,
         default_filter => $self->{default_in_filter},
@@ -563,7 +572,6 @@ sub execute {
             from2 => $self->type_rule->{from2}
         },
     );
-    $result;
 }
 
 sub get_table_info {
@@ -1187,18 +1195,7 @@ sub values_clause {
     my $wrap = $opts->{wrap} || {};
     
     # Create insert parameter tag
-    my $safety = $self->{safety_character} || $self->safety_character;
-    my $qp = $self->q('');
-    my $q = substr($qp, 0, 1) || '';
-    my $p = substr($qp, 1, 1) || '';
-    
-    # Check unsafety keys
-    unless ((join('', keys %$param) || '') =~ /^[$safety\.]+$/) {
-        for my $column (keys %$param) {
-            croak qq{"$column" is not safety column name } . _subname
-              unless $column =~ /^[$safety\.]+$/;
-        }
-    }
+    my ($q, $p) = split //, $self->q('');
     
     # values clause(performance is important)
     '(' .
@@ -1247,35 +1244,44 @@ sub _create_query {
     unless ($query) {
 
         # Create query
-        my $builder = $self->query_builder;
-        $query = $builder->build_query($source);
+        if (exists $ENV{DBIX_CUSTOM_TAG_PARSE} && !$ENV{DBIX_CUSTOM_TAG_PARSE}) {
+            $source ||= '';
+            my $columns = [];
+            my $c = $self->{safety_character} || $self->safety_character;
+            my %duplicate;
+            my $duplicate;
+            # Parameter regex
+            $source =~ s/([0-9]):/$1\\:/g;
+            while ($source =~ /(^|.*?[^\\]):([$c\.]+)(?:\{(.*?)\})?(.*)/sg) {
+                push @$columns, $2;
+                $duplicate = 1 if ++$duplicate{$columns->[-1]} > 1;
+                $source = defined $3 ? "$1$2 $3 ?$4" : "$1?$4";
+            }
+            $source =~ s/\\:/:/g if index($source, "\\:") != -1;
 
-        # Remove reserved word quote
-        if (my $q = $self->_quote) {
-            $q = quotemeta($q);
-            $_ =~ s/[$q]//g for @{$query->columns}
+            # Create query
+            $query = {sql => $source, columns => $columns, duplicate => $duplicate};
         }
-
+        else {
+            $query = $self->query_builder->build_query($source);
+        }
+        
         # Save query to cache
         $self->cache_method->(
             $self, $source,
             {
-                sql     => $query->sql, 
-                columns => $query->columns,
+                sql     => $query->{sql}, 
+                columns => $query->{columns},
                 tables  => $query->{tables} || []
             }
         ) if $cache;
     }
 
     # Filter SQL
-    if ($after_build_sql) {
-        my $sql = $query->sql;
-        $sql = $after_build_sql->($sql);
-        $query->sql($sql);
-    }
-        
+    $query->{sql} = $after_build_sql->($query->{sql}) if $after_build_sql;
+    
     # Save sql
-    $self->last_sql($query->sql);
+    $self->{last_sql} = $query->{sql};
     
     # Prepare statement handle
     my $sth;
@@ -1287,10 +1293,10 @@ sub _create_query {
     }
     
     # Set statement handle
-    $query->sth($sth);
+    $query->{sth} = $sth;
     
     # Set filters
-    $query->{filters} = $self->filters;
+    $query->{filters} = $self->{filters} || $self->filters;
     
     return $query;
 }
@@ -2619,12 +2625,11 @@ and before type rule filter is executed.
 
     query => 1
 
-C<execute> method return L<DBIx::Custom::Query> object, not executing SQL.
-You can check SQL, column, or get statment handle.
+C<execute> method return hash reference which contain SQL and column
+infromation
 
-    my $sql = $query->sql;
-    my $sth = $query->sth;
-    my $columns = $query->columns;
+    my $sql = $query->{sql};
+    my $columns = $query->{columns};
     
 =item C<reuse>
     
@@ -3478,6 +3483,8 @@ L<DBIx::Custom::Model>
     type # will be removed at 2017/1/1
 
 L<DBIx::Custom::Query>
+
+This module is DEPRECATED! # will be removed at 2017/1/1
     
     # Attribute methods
     default_filter # will be removed at 2017/1/1
